require 'fileutils'
require "jwt"
require 'json'

module ServiceLayerRest
    class ApplicationController < ActionController::Base
        include ServiceLayerHelper
        include Constants
        include ResponseHelper
        include Dll
        
        protect_from_forgery with: :exception

        before_action :check_auth, :except => [:login, :get_clients]
        before_action :check_current_user, :except => [:login, :get_clients]

        def check_auth
            token_used, user_data = check_for_token_and_user_data
                        
            if token_used
                if user_data.present? && user_data[Constants::ID_SERVER_KEY] && user_data[Constants::AUTHTYPE_KEY]
                    user = User.new(user_data)
                    if user.isMicos
                        user_id = user.iD_MicosBenutzer
                    else
                        user_id = user.iD_BVBenutzer
                    end

                    if user_id > -1
                        response_string, string_pointer = ServiceLayerDll.check_token(user_id) 
                        ServiceLayerDll.free_string(string_pointer)

                        active_and_valid, state = extract_data_and_state_from_response_string(response_string)

                        if state == Rack::Utils.status_code(:ok) && active_and_valid
                            sign_in :user, user
                            return 
                        end
                    end
                end
            elsif
                authenticate_or_request_with_http_basic do |username, password|
                    header_params = check_header_params

                    unless header_params.empty?
                        response_string, string_pointer = ServiceLayerDll.check_auth(
                            username, 
                            password, 
                            header_params[Constants::AUTHTYPE_KEY], 
                            header_params[Constants::MANDANT_ID_KEY].to_i, 
                            header_params[ServiceLayerHelper::WINUSER_KEY], 
                            header_params[ServiceLayerHelper::WINCOMPUTER_KEY]
                        )
                        ServiceLayerDll.free_string(string_pointer)

                        response = Oj.load(response_string)
                        auth_params = ServiceLayerHelper::check_response_params(response)
                        
                        unless auth_params.empty?
                            user_data = auth_params[ServiceLayerHelper::USERDATA_KEY]
        
                            user = User.new(user_data)
                            user.id = user_data[Constants::ID_CLIENT_KEY]
        
                            sign_in :user, user
                            return
                        end                        
                    end
                end
            end        

            Rails.logger.error 'Auth Error => ' + response_string.force_encoding("utf-8") if response_string && !response_string.empty? 
        end

        def check_current_user
            return render bad_params_for_render('anwender') if current_user.nil?
        end

        private
        def check_for_token_and_user_data
            token_used = false
            if request && request.headers['Authorization'] 
                data = request.headers['Authorization'].split(" ")

                if data.length == 2 && data[0] == 'Bearer' && data[1].length && Rails.application.config.token_secret
                    token_secret = Rails.application.config.token_secret
                    token_used = true

                    begin
                        token_data = JWT.decode(data[1], token_secret)

                        # TODO: JWT.decode already checks exp, next if statement could be "smaller"
                        if token_data.length == 2 && !token_data[0].empty? && 
                            token_data[0]['exp'] && token_data[0]['exp'] > Time.now.to_i &&
                            !token_data[0]['credentials_hash'].empty? && 
                            !token_data[0]['credentials_hash'][ServiceLayerHelper::USERDATA_KEY].empty?

                            #puts 'protocol => ' + token_data[0]['credentials_hash']['protocol']
                            #puts 'base_url => ' + token_data[0]['credentials_hash']['base_url']
                            
                            return [token_used, token_data[0]['credentials_hash'][ServiceLayerHelper::USERDATA_KEY]]
                        end   
                    rescue
                        # Silently ignore errors
                    end              
                end
            end   

            [token_used, nil]
        end
    end
end
