module ServiceLayerRest::Dll
    extend ActiveSupport::Concern
    extend self
    include ServiceLayerHelper
    
    included do
        # Init Dll Gem
        before_action :init_service_layer
    end

    def expected_version
        begin
            file_path = File.dirname(__FILE__)+"/../../../DllVersion.txt"
            file_line = File.read(file_path).strip
        
            [true, file_line]
        rescue => ex 
            [false, ex.message]            
        end
    end
    
    def version
        begin
            file_path = File.dirname(__FILE__)+"/../../../target-svn-data.json"
            file_data = JSON.parse(File.read(file_path))
        
            [true, file_data]
        rescue => ex 
            [false, ex.message]            
        end
    end

    def get_version
        version_state, version_data = version

        begin
            render create_json_error_response("#{ServiceLayerRest::Constants::ERROR_TEXT} DLL Version - #{version_data}") unless version_state
            
            response_string, string_pointer = ServiceLayerDll.get_system_info_and_pep_ini()
            ServiceLayerDll.free_string(string_pointer)

            response = JSON.parse(response_string)

            render ServiceLayerHelper::create_json_response(:ok, ServiceLayerHelper::THIS_SOURCE, ServiceLayerRest::ResponseHelper::NO_SOURCE_DETAILS, {
                'version_data': version_data,
                'backend_info': response,
                'gems': {
                    'service_layer_helper': Gem.loaded_specs['service_layer_helper'] ? Gem.loaded_specs['service_layer_helper'].version.version : 'N/A',
                    'service_layer_rest': Gem.loaded_specs['service_layer_rest'] ? Gem.loaded_specs['service_layer_rest'].version.version : 'N/A',
                    'service_layer_dll': Gem.loaded_specs['service_layer_dll'] ? Gem.loaded_specs['service_layer_dll'].version.version : 'N/A',
                    'token_server': Gem.loaded_specs['token_server'] ? Gem.loaded_specs['token_server'].version.version : 'N/A'
                },
                'Rails.root': Rails.root,
                'logger': Rails.logger.inspect
            })
        rescue => ex 
            render create_json_error_response("#{ServiceLayerRest::Constants::ERROR_TEXT} DLL Version - 'get_version' => '" + ex.message + "'")           
        end
    end

    def init_service_layer
        version_state, version = expected_version

        if version_state
            begin
                ServiceLayerDll.init version
            rescue => ex 
                render create_json_error_response(ex.message)           
            end
        else
            render create_json_error_response("#{ServiceLayerRest::Constants::ERROR_TEXT} DLL Version - #{version}")             
        end
    end
end
