require 'fileutils'
require 'reverse_proxy'

module ServiceLayerRest
    class ReportsController < ApplicationController
        include ServiceLayerHelper
        include Base
        include Dll
        include ReverseProxy::Controller

        def server
            begin
                query_string = params[:params] ? '?' + { 'params': params[:params].permit! }.to_query : ''
                path = "/api/v1/reports/"+ (params[:path] ? params[:path] : '') + query_string
            
                #TODO: Check timeouts!!!
                reverse_proxy Rails.application.config.reportserver_url, verify_ssl: Rails.env.production?, path: path, http: { read_timeout: 20, open_timeout: 100 } do |config|
                    # Curr. just do nothing :-)
                    config.on_error do |code, response|
                        # puts 'ERROR' + response.body
                    end
                end
            rescue => exception
                # puts exception.message
            end
        end

        def is_authorized
            invalid_key = check_params([:reportid], [])
            return render bad_params_for_render(invalid_key) unless invalid_key.blank?

            render ServiceLayerHelper::create_json_response(:ok, ServiceLayerHelper::THIS_SOURCE, "", true)
        end

        # TODO: Need a special techn. user in the backend (how to test him?)
        def register
            invalid_key = check_params([:reportid, :title], [])
            return render bad_params_for_render(invalid_key) unless invalid_key.blank?

            render ServiceLayerHelper::create_json_response(:ok, ServiceLayerHelper::THIS_SOURCE, "", true)
        end

        def configurations
            invalid_key = check_params([:reportid], [:employee_id])
            return render bad_params_for_render(invalid_key) unless invalid_key.blank?

            data = get_configuration(params[:reportid].to_i)

            render ServiceLayerHelper::create_json_response(:ok, ServiceLayerHelper::THIS_SOURCE, data, true)
        end

        def employee_statistic
            invalid_key = check_params([:reportid, :from, :to], [:employee_id, :planarea_id, :configuration_id])
            return render bad_params_for_render(invalid_key) unless invalid_key.blank?

            data = 
            {
                "working_hour_vacation_statistics": 
                {
                    "Ist_Std": "-1,5",
                    "Fe_Gut": "-1,5",
                    "Soll": "-1,5",
                    "Diff": "-1,5",
                    "Korr": "-1,5",
                    "Bezahlt": "-1,5",
                    "Vortrag": "-1,5",
                    "BD_FZA": "-1,5",
                    "fakt": "-1,5",
                    "Uebertr": "-1,5",
                    "Urlaub_Tage": "-1,5",
                    "Urlaub_Tage_gesamt": "-1,5",
                    "Krank": "-1,5",
                    "AZV": "-1,5",
                    "TgPlan": "-1,5",
                    "StdWo": "-1,5",
                    "TgWo": "-1,5",
                    "StdTag": "-1,5",
                    "StdPlan": "-1,5",
                    "Nacht": "-1,5",
                    "Sa": "-1,5",
                    "So": "-1,5",
                    "Ue_Zu": "-1,5",
                    "Vorfeier": "-1,5",
                    "Feiertag": "-1,5",
                    "Feiertag_bez": "-1,5",
                    "So_Feier": "-1,5",
                    "So_Feier_Bez": "-1,5",
                    "Urlaub_Jahr": "-1,5",
                    "Tage": "-1,5",
                    "gen": "-1,5",
                    "gepl": "-1,5",
                    "kap": "-1,5",
                    "kor": "-1,5",
                    "Rest": "-1,5",
                    "StdLZK": "-1,5",
                    "Bereitschaft": "-1,5",
                    "Bereitschaft_Freizeit": "-1,5",
                    "Bereitschaft_Bezahlt": "-1,5",
                    "Rufdienst": "-1,5",
                    "Rufdienst_Freizeit": "-1,5",
                    "Rufdienst_bezahlt": "-1,5",
                    "Einsatz": "-1,5",
                    "Einsatz_Freizeit": "-1,5",
                    "Einsatz_bezahlt": "-1,5",
                    "Zuschlaege_Nacht": "-1,5",
                    "Zuschlaege_Samstag": "-1,5",
                    "Zuschlaege_Sonntag": "-1,5",
                    "Zuschlaege_Vorfeier": "-1,5",
                    "Zuschlaege_Feiertag": "-1,5"
                }
            }

            render ServiceLayerHelper::create_json_response(:ok, ServiceLayerHelper::THIS_SOURCE, data, true)
        end
    
        private 
        def get_configuration(reportid)
            if (reportid == 2) or (reportid == 3)
                fake_file_path = File.dirname(__FILE__)+"/configurations_"+reportid.to_s+".json"
                return JSON.parse(File.read(fake_file_path))
            end

            return {}
        end
    end
end
